@@ -182,6 +182,9 @@ except ImportError:
 	from sphinx_autodoc_typehints import _LOGGER as sat_logger
 	from sphinx_autodoc_typehints import _PYDATA_ANNOTATIONS as pydata_annotations
 
+if isinstance(next(iter(pydata_annotations)), tuple):
+	pydata_annotations = {x[1] for x in pydata_annotations}
+
 try:
 	# 3rd party
 	from sphinx_autodoc_typehints import _future_annotations_imported
@@ -232,7 +235,7 @@ split_type_comment_args = sphinx_autodoc_typehints.split_type_comment_args
 
 
 # Demonstration of module default argument in signature
-def serialise(obj: Any, library=json) -> str:  # noqa: MAN001
+def serialise(obj: Any, library=json) -> str:  # type: ignore[empty-body]  # noqa: MAN001
 	"""
 	Serialise an object into a JSON string.
 
@@ -372,6 +375,11 @@ def format_annotation(annotation: Any, fully_qualified: bool = False) -> str:
 
 		args = get_annotation_args(annotation, module, class_name)
 	except ValueError:
+		if isinstance(annotation, str) and '|' in annotation and sys.version_info < (3, 10):
+			sat_logger.warning(
+					"Unsupported | character in type annotation '%s' for this version of Python",
+					annotation,
+					)
 		return f":py:obj:`~.{annotation}`"
 
 	if module == "_io":
@@ -404,10 +412,11 @@ def format_annotation(annotation: Any, fully_qualified: bool = False) -> str:
 		full_name = "typing.Optional"
 	elif full_name == "types.UnionType":
 		full_name = "typing.Union"
-		role = "data"
 	elif full_name == "typing.Callable" and args and args[0] is not ...:
 		formatted_args = "\\[\\[" + ", ".join(format_annotation(arg) for arg in args[:-1]) + ']'
 		formatted_args += ", " + format_annotation(args[-1]) + ']'
+	elif full_name == "collections.abc.Callable":
+		role = "class"
 	elif full_name == "typing.Literal":
 		# TODO: Enums?
 		formatted_arg_list: DelimitedList[str] = DelimitedList()
@@ -419,6 +428,9 @@ def format_annotation(annotation: Any, fully_qualified: bool = False) -> str:
 
 		formatted_args = f"\\[{formatted_arg_list:, }]"
 
+	if full_name == "typing.Union":
+		role = "class"  # With at least the 3.14 docs.
+
 	if full_name == "typing.Optional":
 		args = tuple(x for x in args if x is not type(None))  # noqa: E721
 
@@ -519,7 +531,7 @@ def preprocess_class_defaults(
 			else:
 				if hasattr(obj, "__attrs_attrs__") and default is attr.NOTHING:
 					# Special casing for attrs classes
-					for value in obj.__attrs_attrs__:  # type: ignore[attr-defined]
+					for value in obj.__attrs_attrs__:
 						if value.name == argname:
 							if isinstance(value.default, attr.Factory):  # type: ignore[arg-type]
 								default = value.default.factory()
@@ -904,7 +916,7 @@ def setup(app: Sphinx) -> SphinxExtMetadata:
 
 
 def _resolve_forwardref(
-		fr: Union[ForwardRef, sphinx.util.typing.ForwardRef],
+		fr: Union[ForwardRef, "sphinx.util.typing.ForwardRef"],
 		module: str,
 		) -> object:
 	"""
@@ -924,4 +936,4 @@ def _resolve_forwardref(
 	elif sys.version_info >= (3, 12):
 		return fr._evaluate(module_dict, module_dict, set(), recursive_guard=set())
 	else:
-		return fr._evaluate(module_dict, module_dict, set())
+		return fr._evaluate(module_dict, module_dict, set())  # type: ignore[arg-type]
